(***************************************************************************
 *   Copyright (C) 2007  Antoine Bodin <antoinexp@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        * 
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************)
 
(** Some functions included by default *)


open Num
open Core


(** {Some usefull functions} *)


let one_var f = function [x] -> f x | _ -> failwith "expected one variable"
let two_var f = function [x;y] -> f x y | _ -> failwith "expected two variables"
let three_var f = function [x;y;z] -> f x y z | _ -> failwith "expected three variables"


let get_int =
  function
      Int i -> i
    | _ -> raise Not_eval


let get_float =
  function
      Float i -> i
    | Int i -> float_of_num i
    | _ -> raise Not_eval


let get_string =
  function
      String s -> s
    | _ -> raise Not_eval


let caml_float_float f = one_var (fun x -> Float( f (get_float x)))  (* very usefull for cos,exp,sin,etc *)


let get_int_interval =
  function
      Op (Interval, Int i, Int j) -> (i,j)
    | _ -> raise Not_eval


let fun_of_expr l expr var x =  
  match var with
      Var name -> Eval.evalfun l (Fun ([name],expr),x)
    | _ -> raise Not_eval


(** {Functions loaded by default in matl} *)


let min l = List.hd l (* TODO *)


let list_var l _ = 
  let sq = Hashtbl.fold
    (fun s -> fun _ -> fun sq -> String s::sq) l []
  in Seq sq


let del_var l x = 
  Variable.remove l (get_string x); 
  Void


let _mod x y = Int (mod_num (get_int x) (get_int y))


let seq l expr cond =
  let var,int =
    match cond with
	LOp (Eq, var, int) -> var,int
      | _ -> failwith "you should write: var=i..j"
  in let i,j = get_int_interval int in
  let i,j = int_of_num i, int_of_num j in
  let f = fun_of_expr l expr var in
  let t = Array.init (j-i+1) (fun k -> f (Int (num_of_int (k+i)))) in
    Seq (Array.to_list t)


let sum l expr cond =
  let var,int =
    match cond with
	LOp (Eq, var, int) -> var,int
      | _ -> failwith "you should write: var=i..j"
  in let i,j = get_int_interval int in
  let i,j = int_of_num i, int_of_num j in
  let f = fun_of_expr l expr var in
  let r = ref (Int (Num.Int 0)) in
    for k=0 to (j-i) do
      r:=Eval.evalexpr l (Op (Add, !r, f (Int (num_of_int (k+i)))))
    done; (!r)


(** {The laoding function} *)  


let load l =
  List.iter (fun (s,f) -> Variable.set l s (INTFun f))
    [("min",min);
     ("list_var",list_var l);
     ("del_var",one_var (del_var l));
     ("cos",caml_float_float cos);
     ("acos",caml_float_float acos);
     ("sin",caml_float_float sin);
     ("asin",caml_float_float asin);
     ("tan",caml_float_float tan);
     ("atan",caml_float_float atan);
     ("exp",caml_float_float exp);
     ("sqrt",caml_float_float sqrt);
     ("mod",two_var _mod);
     ("evalf",one_var (fun x -> Float (get_float x)));
     ("seq",two_var (seq l));
     ("sum",two_var (sum l))
    ]
  
