(***************************************************************************
 *   Copyright (C) 2007  Antoine Bodin <antoinexp@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        * 
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************)
 
(** Compute the matl expressions *)


open Num
open Core


let rec pow x =
  let rec f =
    function
	0 -> 1
      | n -> x*(f (n-1))
  in f
   

let apply_op f (+) (+.)  =
  function
      (Int i, Int j) -> Int (i+j)
    | (Float i, Float j) -> Float (i +. j)
    | (Int i, Float j) -> Float ((float_of_num i)+.j)
    | (Float i, Int j) -> Float (i+.(float_of_num j))
    | x -> f x


let eval_op op =
  let id = fun (i,j) -> Op (op,i,j) in
  let f = apply_op id in
    match op with
	Add -> f (+/) (+.)
      | Sub -> f (-/) (-.)
      | Mult -> f ( */ ) ( *. )
      | Div -> f (//) (/.)
      | Pow -> f ( **/  ) ( ** )
      | Interval -> id


let apply_lop f (<) (<.)  =
  function
      (Int i, Int j) -> Bool (i<j)
    | (Float i, Float j) -> Bool (i<.j)
    | (Int i, Float j) -> Bool ((float_of_num i)<.j)
    | (Float i, Int j) -> Bool (i<.(float_of_num j))
    | x -> f x


let apply_llop f (||) =
  function
      (Bool i, Bool j) -> Bool (i || j)
    | x -> f x


let eval_lop lop =
  let id = fun (i,j) -> LOp (lop,i,j) in
  let f = apply_lop id in
  let g = apply_llop id in
    match lop with
	Eq -> f (=/) (=)
      | Neq -> f (<>/) (<>)
      | Low -> f (</) (<)
      | Gre -> f (>/) (>)
      | Leq -> f (<=/) (<=)
      | Geq -> f (>=/) (>=)
      | Lor -> g (||)
      | Land -> g (&&)
      | _ -> (fun (i,j) -> LOp (lop,i,j)) (* TODO: Lnot, Land, Lor *)
