(***************************************************************************
 *   Copyright (C) 2007  Antoine Bodin <antoinexp@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        * 
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************)
 
(** The main types used in matl *)


type operator = Add | Mult | Pow | Sub | Div | Interval
type loperator = Eq|Neq|Low|Gre|Leq|Geq|Lnot|Land|Lor
type varname = string


type expr =
      Op of operator*expr*expr
    | LOp of loperator*expr*expr
    | Var of varname
    | Int of Num.num
    | Bool of bool
    | Float of float
    | String of string
    | Seq of expr list
    | INTFun of (expr list -> expr)
    | Fun of varname list*expr
    | Call of expr*expr
    | Proc of varname list*seq list
    | Void
and inst =
      For of varname*expr*expr*seq list
    | While of expr*seq list
    | Def of varname*expr
    | If of expr*seq list*seq list (* condition,seq if true, seq else*)
    | Expr of expr
    | Include of string
    | Restart
and seq = {verbose:bool; inst:inst}


module Variable =
  struct
    type t=(varname,expr) Hashtbl.t

    let newtable () = (Hashtbl.create 50: t)
    let clear (l:t) = Hashtbl.clear l
    let copy (l:t) = Hashtbl.copy l

    let remove l name =
      Hashtbl.remove l name

    let set l name expr = 
      Hashtbl.remove l name;
      Hashtbl.add l name expr

    let get l eval name =
      try 
	eval (Hashtbl.find l name)
      with Not_found -> Var name
  end


(* internal *)
exception Not_eval
