(***************************************************************************
 *   Copyright (C) 2007  Antoine Bodin <antoinexp@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        * 
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************)
 
(** Evaluate the matl sequences *)


open Core
open Print
    

let rec evalfun l (fn,value) =
  let sq = match evalexpr l value with Seq sq -> sq | v -> [v] in
  let apply f vl e = 
    let a = List.combine vl sq in
    let l = Variable.copy l in
      List.iter (fun (name,value) -> Variable.set l name value) a;
      f l e
  in match evalexpr l fn with
      Fun (vl,e) -> apply evalexpr vl e
    | Proc (vl,seql) -> apply evalprocseql vl seql
    | INTFun g -> (try g sq with Not_eval -> Call(fn,Seq sq))
    | _ -> Call (fn,Seq sq)


and evalexpr l expr =
  match expr with
      Op (op,x,y) -> Compute.eval_op op (evalexpr l x,evalexpr l y)
    | LOp (lop,x,y) -> Compute.eval_lop lop (evalexpr l x,evalexpr l y)
    | Var name -> Variable.get l (evalexpr l) name (* evaluate the variables under the variable ? *)
    | Seq sq -> Seq (List.map (evalexpr l) sq)
    | Fun (vl,e) -> Fun (vl,evalexpr l e)
    | Call (fn,value) -> evalfun l (fn,value)
    | _ -> expr


and evalinst l inst = 
  let get_int e =
      match evalexpr l e with
	  Int i -> Num.int_of_num i
	| _ -> raise Exit
  in let get_bool e =
    match evalexpr l e with
	Bool b -> b
      | _ -> raise Exit
  in match inst with
      Expr e -> Expr (evalexpr l e)
    | Def (name,expr) -> let e = evalexpr l expr in 
			   Variable.set l name e; Def (name,e)
    | Restart -> Variable.clear l; Restart
    | Include s -> Include s (* TODO *)
    | While (cond,seql) ->
	begin try
	    while (get_bool (evalexpr l cond)) do
	      evalseql l seql;
	    done;
	  with Exit _ -> () end;
	While (cond,seql)
    | For (var,i,j,seql) -> 
	(try
	    for k=get_int i to get_int j do
	      Variable.set l var (Int (Num.num_of_int k));
	      evalseql l seql;
	    done;
	  with Exit _ -> ());
	For (var,i,j,seql)
    | If (expr,etrue,efalse) ->
	let e = evalexpr l expr in
	  (try
	     Expr (evalprocseql l (if get_bool e then etrue else efalse))
	    with Exit _ -> If (e,etrue,efalse))


and evalseql l =
  function
      [] -> ()
    | seq::tail -> 
	let inst = evalinst l seq.inst in
	  if seq.verbose then begin
	      print_inst stdout inst;
	      print_newline ()
	    end;
	 evalseql l tail
	  

and evalprocseql l seql =
  let rec f =
    function
	[] -> None
      | seq::tail -> 
	  (let inst = evalinst l seq.inst in
	    match f tail with
		None ->
		  (match inst with
		      Expr e -> Some e
		    | _ -> None)
	      | x -> x)
  in match (f seql) with Some e -> e | None -> Void
	  
	
