(***************************************************************************
 *   Copyright (C) 2007  Antoine Bodin <antoinexp@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        * 
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************)
 
(** The main functions called to run matl *)


let prompt l inc =
  let tty = Unix.isatty (Unix.descr_of_in_channel inc) in
  if tty then print_endline "\tMatl Version 0.0.1\n";
  let lexing = Lexing.from_channel inc in
  let exiting () = if tty then (Lexing.new_line lexing; Lexing.flush_input lexing) else exit 0 in
    while not !(Lexer.end_requested) do
      if tty then (print_string "[> "; flush stdout);
      try
	let seql = Lexer.parseseql [< Lex.lexer lexing >] in
	  Eval.evalseql l seql
      with Stream.Error s -> (Printf.printf "Matl: Expression error: %s\n" s; exiting ())
	| Lex.Parsing_failure s -> (Printf.printf "Matl: Parsing failure: %s\n" s; exiting ())
	| Lexer.Syntax_error s -> (Printf.printf "Matl: Syntax error: %s\n" s; exiting ())
    done; Lexer.reset (); if tty then print_endline "Matl: Exiting"


let compiling l out inc = 
  let stream = Lexer.stream_from_channel inc in
  let code = ref [] in
    while not !(Lexer.end_requested) do
      try 
	code:=Lexer.parseseql stream::(!code)
      with Stream.Error s -> (Printf.printf "Matl: Expression error: %s\n" s; exit 0)
	| Lex.Parsing_failure s -> (Printf.printf "Matl: Parsing failure: %s\n" s; exit 0)
	| Lexer.Syntax_error s -> (Printf.printf "Matl: Syntax error: %s\n" s; exit 0)
    done; Lexer.reset (); Marshal.to_channel out (List.rev (!code)) []


let loadbytecode l inc =
  let code = Marshal.from_channel inc in
    List.iter (Eval.evalseql l) code


let applyfile f path =
    let fd = open_in_bin path in
      f fd; close_in fd


let compile = ref false
let output = ref "out.tlo"
let bytecode = ref false
let files = ref [] 
let _ =
  Arg.parse [
      ("-c", Arg.Set compile, "used to compile the given <files>");
      ("-b", Arg.Set bytecode, "load bytecode <files>");
      ("-o", Arg.Set_string output, "<output> set the output file (out.tlo by default)")
  ] (fun s -> files:=s::(!files)) "Usage: matl <options> <files>\nOptions are:";
  let l = Core.Variable.newtable () in
    Basic.load l;
    if !compile then (
	let fd = open_out_bin (!output) in
	  List.iter (applyfile (compiling l fd)) (!files);
	  close_out fd
    ) else (
	if !files=[] then prompt l stdin
	else List.iter (applyfile ((if !bytecode then loadbytecode else prompt) l)) (!files)
    )

