open Core

(* printing *)

let string_of_operator =
  function
      Add -> "+"
    | Mult -> "*"
    | Pow -> "^"
    | Sub -> "-"
    | Div -> "/"
    | Interval -> ".."

let string_of_loperator =
  function
      Eq -> "="
    | Neq -> "<>"
    | Low -> "<"
    | Gre -> ">"
    | Leq -> "<="
    | Geq -> ">="
    | Lnot -> "not"
    | Land -> "and"
    | Lor -> "or"


let rec print_expr out =
  let print = output_string out in
  let rec print_vl =
    function
	[] -> print "()"
      | [a] -> print "("; print a; print ")"
      | a::b -> print "(";
	  print a;
	  List.iter (fun a -> print ", "; print a) b;
	  print ")"
  and f =
    function
	Op (op,x,y) -> f x; print (string_of_operator op); f y
      | LOp (lop,x,y) -> f x; print (string_of_loperator lop); f y
      | Var name -> print name
      | Float f -> print (string_of_float f)
      | Int i -> print (Num.string_of_num i)
      | Bool b -> print (string_of_bool b)
      | String s -> List.iter print ["\""; s; "\""]
      | INTFun _ -> () (* shouldn't be matched normally... *)
      | Fun (vl,e) -> print_vl vl; print "->"; f e
      | Call (g,e) -> print "("; f g; print ")"; print "("; f e; print ")"
      | Void -> print "()"
      | Proc (vl,seql) -> print "proc"; print_vl vl; print "\n"; 
	  print_seql out seql; print "end proc"
      | Seq l -> 
	  print "(";
	  begin match l with
	      [] -> ()
	    | a::[] -> f a
	    | a::b -> f a;
		List.iter (fun e -> print ", "; f e) b
	  end; print ")"
  in f

and print_inst out =
  let print = output_string out in
  let print_e = print_expr out in
  let rec f =
    function
	Expr e -> print_e e
      | Def (name,expr) -> print name; print " := "; print_e expr
      | Restart -> print "restart;"
      | Include s -> print ("with "^s^";")
      | For (var,fv,tv,seql) ->
	  print "for ";
	  print var;
	  print " from ";
	  print_e fv;
	  print " to ";
	  print_e tv;
	  print " do\n";
	  print_seql out seql;
	  print "\nod;"
      | While (cond,seql) -> 
	  print "while ";
	  print_e cond;
	  print " do\n";
	  print_seql out seql;
	  print " od;"
      | If (cond,seql1,seql2) ->
	  print "if ";
	  print_e cond;
	  print " then\n";
	  print_seql out seql1;
	  print "else\n";
	  print_seql out seql2;
	  print "fi;"
  in f

and print_seql out = 
  List.iter (
      fun seq -> 
	print_inst out seq.inst;
	output_string out "\n" 
  ) 
